@@ -21,21 +21,6 @@
 #ifndef DEFINES_H
 # define DEFINES_H
 
-#ifndef __GNUC__
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
-# else
-#  ifdef _AIX
-#pragma alloca
-#  else
-#   ifndef alloca	/* predefined by HP cc +Olibcalls */
-char *alloca();
-#   endif
-#  endif
-# endif
-#endif
-
-
 #include <config.h>
 
 #ifdef STDC_HEADERS
@@ -46,11 +31,17 @@ char *alloca();
 #endif
 
 #ifdef NO_TIME_T
+# define HAVE_TIME_T
 typedef unsigned int time_t;
 #endif
 
 #ifdef NO_SIZE_T
+# define HAVE_SIZE_T
 typedef unsigned int size_t;
+#endif
+
+#ifdef NO_SSIZE_T
+# define HAVE_SSIZE_T
 typedef int ssize_t;
 #endif
 
@@ -62,6 +53,14 @@ typedef long ptrdiff_t;
 # include <strings.h>
 #endif
 
+#ifndef HAVE_STRNSTR
+char *strnstr(const char *haystack, const char *needle, size_t haystacklen);
+#endif
+
+#ifdef HAVE_STDDEF_H
+# include <stddef.h>
+#endif
+
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -103,6 +102,12 @@ typedef struct {
 	unsigned char i[8];
 } uint64;
 
+#ifndef HAVE_ISASCII
+# ifndef isascii
+#  define isascii(x) (x<128?1:0)
+# endif
+#endif
+
 #if SIZEOF_UNSIGNED_LONG == 4
 typedef unsigned long int uint32;
 typedef signed long int sint32;
