@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
+ *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -46,6 +47,8 @@
 #include <gnutls_alert.h>
 #include <gnutls_state.h>
 #include <ext_srp.h>
+#include <gnutls_rsa_export.h> /* for gnutls_get_rsa_params() */
+#include <auth_anon.h> /* for gnutls_anon_server_credentials */
 
 #ifdef HANDSHAKE_DEBUG
 #define ERR(x, y) _gnutls_handshake_log( "HSK[%x]: %s (%d)\n", session, x,y)
@@ -215,18 +218,21 @@ int _gnutls_finished(gnutls_session session, int type, void *ret)
 int _gnutls_create_random(opaque * dst)
 {
 	uint32 tim;
-	opaque rand[TLS_RANDOM_SIZE - 4];
+
+	/* Use weak random numbers for the most of the
+	 * buffer except for the first 4 that are the
+	 * system's time.
+	 */
 
 	tim = time(NULL);
 	/* generate server random value */
 	_gnutls_write_uint32(tim, dst);
 
 	if (_gnutls_get_random
-	    (rand, TLS_RANDOM_SIZE - 4, GNUTLS_STRONG_RANDOM) < 0) {
+	    (&dst[4], TLS_RANDOM_SIZE - 4, GNUTLS_WEAK_RANDOM) < 0) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	memcpy(&dst[4], rand, TLS_RANDOM_SIZE - 4);
 
 	return 0;
 }
@@ -265,6 +271,12 @@ int _gnutls_read_client_hello(gnutls_session session, opaque * data,
 		 * then we send him the highest we support.
 		 */
 		ver = _gnutls_version_max(session);
+		if (ver==GNUTLS_VERSION_UNKNOWN) {
+			/* this check is not really needed.
+			 */
+			gnutls_assert();
+			return GNUTLS_E_UNKNOWN_CIPHER_SUITE;
+		}
 	} else {
 		ver = version;
 	}
@@ -495,14 +507,21 @@ int _gnutls_recv_finished(gnutls_session session)
 }
 
 /* returns PK_RSA if the given cipher suite list only supports,
- * RSA algorithms, PK_DSA if DSS, and -1 if both or none.
+ * RSA algorithms, PK_DSA if DSS, and GNUTLS_PK_ANY if both or 
+ * GNUTLS_PK_NONE for none.
  */
-int _gnutls_server_find_pk_algos_in_ciphersuites( opaque* data, int datalen) {
+static int _gnutls_server_find_pk_algos_in_ciphersuites( opaque* data, int datalen) 
+{
 int j;
-gnutls_pk_algorithm algo=-1, prev_algo = 0;
+gnutls_pk_algorithm algo=GNUTLS_PK_NONE, prev_algo = 0;
 gnutls_kx_algorithm kx;
 GNUTLS_CipherSuite cs;
 
+	if (datalen % 2 != 0) {
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+	}
+
 	for (j = 0; j < datalen; j += 2) {
 		memcpy( &cs.CipherSuite, &data[j], 2);
 		kx = _gnutls_cipher_suite_get_kx_algo( cs);
@@ -510,7 +529,7 @@ GNUTLS_CipherSuite cs;
 		if ( _gnutls_map_kx_get_cred( kx, 1) == GNUTLS_CRD_CERTIFICATE) {
 			algo = _gnutls_map_pk_get_pk( kx);
 	
-			if (algo!=prev_algo && prev_algo!=0) return -1;
+			if (algo!=prev_algo && prev_algo!=0) return GNUTLS_PK_ANY;
 			prev_algo = algo;
 		}
 	}
@@ -551,6 +570,11 @@ int _gnutls_server_select_suite(gnutls_session session, opaque *data, int datale
 		else return GNUTLS_E_UNKNOWN_CIPHER_SUITE;
 	}
 
+	if (datalen % 2 != 0) {
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+	}
+
 #ifdef HANDSHAKE_DEBUG
 
 	_gnutls_handshake_log("HSK[%x]: Requested cipher suites: \n", session);
@@ -568,7 +592,7 @@ int _gnutls_server_select_suite(gnutls_session session, opaque *data, int datale
 	memset(session->security_parameters.current_cipher_suite.CipherSuite, '\0', 2);
 
 	retval = GNUTLS_E_UNKNOWN_CIPHER_SUITE;
-	
+
 	for (j = 0; j < datalen; j += 2) {
 		for (i = 0; i < x; i++) {
 			if (memcmp(ciphers[i].CipherSuite, &data[j], 2) ==
@@ -1200,11 +1224,9 @@ opaque buf[2*TLS_MAX_SESSION_ID_SIZE+1];
 	_gnutls_handshake_log("HSK[%x]: SessionID: %s\n", session,
 		    _gnutls_bin2hex(session_id, session_id_len, buf, sizeof(buf)));
 
-	if ((session->internals.resumed_security_parameters.
-	     session_id_size > 0)
-	    && memcmp(session_id,
-		      session->internals.
-		      resumed_security_parameters.session_id,
+	if (session_id_len > 0 &&
+        	session->internals.resumed_security_parameters.session_id_size == session_id_len &&
+		memcmp(session_id, session->internals.resumed_security_parameters.session_id,
 		      session_id_len) == 0) {
 		/* resume session */
 		memcpy(session->internals.
@@ -1219,7 +1241,8 @@ opaque buf[2*TLS_MAX_SESSION_ID_SIZE+1];
 
 		return 0;
 	} else {
-		/* keep the new session id */
+		/* keep the new session id 
+		 */
 		session->internals.resumed = RESUME_FALSE;	/* we are not resuming */
 		session->security_parameters.session_id_size =
 		    session_id_len;
@@ -1279,7 +1302,6 @@ static int _gnutls_read_server_hello(gnutls_session session, opaque *data,
 	}
 	DECR_LEN(len, session_id_len);
 
-
 	/* check if we are resuming and set the appropriate
 	 * values;
 	 */
@@ -1330,7 +1352,7 @@ static int _gnutls_read_server_hello(gnutls_session session, opaque *data,
  * Needed in client hello messages. Returns the new data length.
  */
 static int _gnutls_copy_ciphersuites(gnutls_session session,
-				     opaque ** ret_data)
+				     opaque * ret_data, size_t ret_data_size)
 {
 	int ret, i;
 	GNUTLS_CipherSuite *cipher_suites;
@@ -1372,21 +1394,18 @@ static int _gnutls_copy_ciphersuites(gnutls_session session,
 
 	datalen += sizeof(uint16) + cipher_num;
 
-	*ret_data = gnutls_malloc(datalen);
-	if (*ret_data == NULL) {
+	if ((size_t)datalen > ret_data_size) {
 		gnutls_assert();
-		gnutls_free(cipher_suites);
-		return GNUTLS_E_MEMORY_ERROR;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
-
-	_gnutls_write_uint16(cipher_num, *ret_data);
+	
+	_gnutls_write_uint16(cipher_num, ret_data);
 	pos += 2;
 
 	for (i = 0; i < (cipher_num / 2); i++) {
-		memcpy( &(*ret_data)[pos], cipher_suites[i].CipherSuite, 2);
+		memcpy( &ret_data[pos], cipher_suites[i].CipherSuite, 2);
 		pos += 2;
 	}
-
 	gnutls_free(cipher_suites);
 
 	return datalen;
@@ -1397,7 +1416,7 @@ static int _gnutls_copy_ciphersuites(gnutls_session session,
  * Needed in hello messages. Returns the new data length.
  */
 static int _gnutls_copy_comp_methods(gnutls_session session,
-				     opaque ** ret_data)
+				     opaque * ret_data, size_t ret_data_size)
 {
 	int ret, i;
 	uint8 *compression_methods, comp_num;
@@ -1416,17 +1435,15 @@ static int _gnutls_copy_comp_methods(gnutls_session session,
 	datalen = pos = 0;
 	datalen += comp_num + 1;
 
-	*ret_data = gnutls_malloc(datalen);
-	if (*ret_data == NULL) {
+	if ((size_t)datalen > ret_data_size) {
 		gnutls_assert();
-		gnutls_free(compression_methods);
-		return GNUTLS_E_MEMORY_ERROR;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
-	(*ret_data)[pos++] = comp_num; /* put the number of compression methods */
+	ret_data[pos++] = comp_num; /* put the number of compression methods */
 
 	for (i = 0; i < comp_num; i++) {
-		(*ret_data)[pos++] = compression_methods[i];
+		ret_data[pos++] = compression_methods[i];
 	}
 
 	gnutls_free(compression_methods);
@@ -1434,17 +1451,22 @@ static int _gnutls_copy_comp_methods(gnutls_session session,
 	return datalen;
 }
 
+/* This should be sufficient by now. It should hold all the extensions
+ * plus the headers in a hello message.
+ */
+#define MAX_EXT_DATA_LENGTH 1024
+
 /* This function sends the client hello handshake message.
  */
 static int _gnutls_send_client_hello(gnutls_session session, int again)
 {
 	opaque *data = NULL;
-	opaque *extdata = NULL;
 	int extdatalen;
 	int pos = 0;
 	int datalen = 0, ret = 0;
 	opaque random[TLS_RANDOM_SIZE];
 	gnutls_protocol_version hver;
+	opaque extdata[MAX_EXT_DATA_LENGTH];
 
 	opaque *SessionID =
 	    session->internals.resumed_security_parameters.session_id;
@@ -1478,12 +1500,10 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 			    resumed_security_parameters.version;
 		}
 
-		if (hver <= 0) {
-			if (hver == 0)
-				hver = GNUTLS_E_INTERNAL_ERROR;
+		if (hver == GNUTLS_VERSION_UNKNOWN || hver == 0) {
 			gnutls_assert();
 			gnutls_free(data);
-			return hver;
+			return GNUTLS_E_INTERNAL_ERROR;
 		}
 
 		data[pos++] = _gnutls_version_get_major(hver);
@@ -1528,18 +1548,16 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 
 		/* Copy the ciphersuites.
 		 */
-		extdatalen = _gnutls_copy_ciphersuites(session, &extdata);
+		extdatalen = _gnutls_copy_ciphersuites(session, extdata, sizeof(extdata));
 		if (extdatalen > 0) {
 			datalen += extdatalen;
 			data = gnutls_realloc_fast(data, datalen);
 			if (data == NULL) {
 				gnutls_assert();
-				gnutls_free(extdata);
 				return GNUTLS_E_MEMORY_ERROR;
 			}
 
 			memcpy(&data[pos], extdata, extdatalen);
-			gnutls_free(extdata);
 			pos += extdatalen;
 
 		} else {
@@ -1553,18 +1571,16 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 
 		/* Copy the compression methods.
 		 */
-		extdatalen = _gnutls_copy_comp_methods(session, &extdata);
+		extdatalen = _gnutls_copy_comp_methods(session, extdata, sizeof(extdata));
 		if (extdatalen > 0) {
 			datalen += extdatalen;
 			data = gnutls_realloc_fast(data, datalen);
 			if (data == NULL) {
 				gnutls_assert();
-				gnutls_free(extdata);
 				return GNUTLS_E_MEMORY_ERROR;
 			}
 
 			memcpy(&data[pos], extdata, extdatalen);
-			gnutls_free(extdata);
 			pos += extdatalen;
 
 		} else {
@@ -1578,19 +1594,17 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 		/* Generate and copy TLS extensions.
 		 */
 		if (hver >= GNUTLS_TLS1) {
-			extdatalen = _gnutls_gen_extensions(session, &extdata);
+			extdatalen = _gnutls_gen_extensions(session, extdata, sizeof(extdata));
 
 			if (extdatalen > 0) {
 				datalen += extdatalen;
 				data = gnutls_realloc_fast(data, datalen);
 				if (data == NULL) {
 					gnutls_assert();
-					gnutls_free(extdata);
 					return GNUTLS_E_MEMORY_ERROR;
 				}
 
 				memcpy(&data[pos], extdata, extdatalen);
-				gnutls_free(extdata);
 			} else if (extdatalen < 0) {
 				gnutls_assert();
 				gnutls_free(data);
@@ -1610,7 +1624,7 @@ static int _gnutls_send_client_hello(gnutls_session session, int again)
 static int _gnutls_send_server_hello(gnutls_session session, int again)
 {
 	opaque *data = NULL;
-	opaque *extdata = NULL;
+	opaque extdata[MAX_EXT_DATA_LENGTH];
 	int extdatalen;
 	int pos = 0;
 	int datalen, ret = 0;
@@ -1628,7 +1642,9 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 		_gnutls_cipher_suite_get_kx_algo(
 			session->security_parameters.current_cipher_suite)))
 	{
-		if (session->security_parameters.extensions.srp_username[0] == 0) {
+		if (session->internals.resumed == RESUME_FALSE &&
+			session->security_parameters.extensions.srp_username[0] == 0) 
+		{
 			/* The peer didn't send a valid SRP extension with the
 			 * SRP username. The draft requires that we send an
 			 * alert and start the handshake again.
@@ -1647,7 +1663,7 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 
 	if (again == 0) {
 		datalen = 2 + session_id_len + 1 + TLS_RANDOM_SIZE + 3;
-		extdatalen = _gnutls_gen_extensions(session, &extdata);
+		extdatalen = _gnutls_gen_extensions(session, extdata, sizeof(extdata));
 
 		if (extdatalen < 0) {
 			gnutls_assert();
@@ -1657,7 +1673,6 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 		data = gnutls_alloca(datalen + extdatalen);
 		if (data == NULL) {
 			gnutls_assert();
-			gnutls_free(extdata);
 			return GNUTLS_E_MEMORY_ERROR;
 		}
 
@@ -1698,7 +1713,6 @@ static int _gnutls_send_server_hello(gnutls_session session, int again)
 			datalen += extdatalen;
 
 			memcpy(&data[pos], extdata, extdatalen);
-			gnutls_free(extdata);
 		}
 	}
 
@@ -1775,7 +1789,7 @@ int _gnutls_recv_hello(gnutls_session session, opaque *data, int datalen)
 
 /**
   * gnutls_rehandshake - This function will renegotiate security parameters
-  * @session: is a a &gnutls_session structure.
+  * @session: is a &gnutls_session structure.
   *
   * This function will renegotiate security parameters with the
   * client. This should only be called in case of a server.
@@ -1816,7 +1830,9 @@ int gnutls_rehandshake(gnutls_session session)
 	return 0;
 }
 
-static int _gnutls_abort_handshake( gnutls_session session, int ret) {
+inline 
+static int _gnutls_abort_handshake( gnutls_session session, int ret) 
+{
 	if ( ((ret==GNUTLS_E_WARNING_ALERT_RECEIVED) && 
 		( gnutls_alert_get(session) == GNUTLS_A_NO_RENEGOTIATION))
 		|| ret==GNUTLS_E_GOT_APPLICATION_DATA)
@@ -1893,10 +1909,11 @@ int gnutls_handshake(gnutls_session session)
 	}
 	if (ret < 0) {
 		/* In the case of a rehandshake abort
-		 * we should reset the handshake's session
+		 * we should reset the handshake's internal state.
 		 */
 		if (_gnutls_abort_handshake( session, ret) == 0)
 			STATE = STATE0;
+
 		return ret;
 	}
 	
@@ -1905,6 +1922,7 @@ int gnutls_handshake(gnutls_session session)
 	if (ret < 0) {
 		if (_gnutls_abort_handshake( session, ret) == 0)
 			STATE = STATE0;
+		
 		return ret;
 	}
 	
@@ -2332,6 +2350,76 @@ int _gnutls_recv_hello_request(gnutls_session session, void *data,
 	}
 }
 
+/* Returns 1 if the given KX has not the corresponding parameters
+ * (DH or RSA) set up. Otherwise returns 0.
+ */
+inline static int check_server_params( gnutls_session session, gnutls_kx_algorithm kx,
+	gnutls_kx_algorithm* alg, int alg_size)
+{
+	int cred_type;
+	const gnutls_certificate_credentials x509_cred;
+	const gnutls_anon_server_credentials anon_cred;
+	gnutls_dh_params dh_params = NULL;
+	gnutls_rsa_params rsa_params = NULL;
+	int j, keep;
+
+	cred_type = _gnutls_map_kx_get_cred( kx, 1);
+	
+	/* Read the DH parameters if any.
+	 */
+	if (cred_type == GNUTLS_CRD_CERTIFICATE) {
+		x509_cred =
+		    _gnutls_get_cred(session->key, cred_type, NULL);
+	
+		if (x509_cred != NULL) {
+			dh_params = _gnutls_certificate_get_dh_params(x509_cred, session);
+			rsa_params = _gnutls_certificate_get_rsa_params(x509_cred, session);
+		}
+
+		/* Check also if the certificate supports the
+		 * KX method.
+		 */
+		keep = 1;
+		for (j = 0; j < alg_size; j++) {
+			if (alg[j] == kx) {
+				keep = 0;
+				break;
+			}
+		}
+		
+		if (keep == 1) return 1;
+
+
+	} else if ( cred_type == GNUTLS_CRD_ANON) {
+		anon_cred =
+		    _gnutls_get_cred(session->key, cred_type, NULL);
+	
+#ifdef ENABLE_ANON
+		if (anon_cred != NULL) {
+			dh_params = _gnutls_anon_get_dh_params(anon_cred, session);
+		}
+#endif
+	} else return 0; /* no need for params */
+
+
+	/* If the key exchange method needs RSA or DH params,
+	 * but they are not set then remove it.
+	 */
+	if (_gnutls_kx_needs_rsa_params( kx) != 0) {
+		/* needs rsa params. */
+		if (_gnutls_get_rsa_params( rsa_params)==NULL)
+			return 1;
+	}
+	
+	if (_gnutls_kx_needs_dh_params( kx) != 0) {
+		/* needs DH params. */
+		if (_gnutls_get_dh_params( dh_params)==NULL)
+			return 1;
+	}
+
+	return 0;
+}
+
 /* This function will remove algorithms that are not supported by
  * the requested authentication method. We remove an algorithm if
  * we have a certificate with keyUsage bits set.
@@ -2347,13 +2435,12 @@ int _gnutls_remove_unwanted_ciphersuites(gnutls_session session,
 
 	int ret = 0;
 	GNUTLS_CipherSuite *newSuite, cs;
-	int newSuiteSize = 0, i, j, keep;
+	int newSuiteSize = 0, i, keep;
 	const gnutls_certificate_credentials x509_cred;
-	const gnutls_cert* cert = NULL;
-	gnutls_kx_algorithm *alg;
-	int alg_size;
 	gnutls_kx_algorithm kx;
 	int server = session->security_parameters.entity==GNUTLS_SERVER?1:0;
+	gnutls_kx_algorithm *alg;
+	int alg_size;
 
 	/* if we should use a specific certificate, 
 	 * we should remove all algorithms that are not supported
@@ -2367,28 +2454,24 @@ int _gnutls_remove_unwanted_ciphersuites(gnutls_session session,
 	/* if x509_cred==NULL we should remove all X509 ciphersuites
 	 */
 
-	cert = NULL;
-	if (session->security_parameters.entity == GNUTLS_SERVER) {
-		cert = _gnutls_server_find_cert(session, requested_pk_algo);
-	}
-
-	if (cert == NULL) {
-		/* No certificate was found 
-		 */
-		alg_size = 0;
-		alg = NULL;
-	} else {
-		/* get all the key exchange algorithms that are 
-		 * supported by the X509 certificate parameters.
-		 */
-		if ((ret =
-		     _gnutls_cert_supported_kx(cert, &alg,
-					       &alg_size)) < 0) {
+	if (session->security_parameters.entity == GNUTLS_SERVER && x509_cred != NULL) {
+		ret = _gnutls_server_select_cert(session, requested_pk_algo);
+		if (ret < 0) {
 			gnutls_assert();
 			return ret;
 		}
 	}
 
+	/* get all the key exchange algorithms that are 
+	 * supported by the X509 certificate parameters.
+	 */
+	if ((ret =
+	     _gnutls_selected_cert_supported_kx(session, &alg,
+				       &alg_size)) < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
 	newSuite =
 	    gnutls_malloc(numCipherSuites * sizeof(GNUTLS_CipherSuite));
 	if (newSuite == NULL) {
@@ -2410,28 +2493,14 @@ int _gnutls_remove_unwanted_ciphersuites(gnutls_session session,
 		 */
 		if (_gnutls_get_kx_cred(session, kx, NULL) == NULL) {
 			keep = 1;
-		} else
-		/* If there was no credentials to use with the specified
-		 * key exchange method, then just remove it.
-		 */
-		if (_gnutls_map_kx_get_cred(kx, server) == GNUTLS_CRD_CERTIFICATE) {
-			keep = 1;	/* do not keep */
+		} else {
+			keep = 0;
 
-			if (x509_cred != NULL) {
-				if (server) {
-					/* here we check if the KX algorithm 
-					 * is compatible with the certificate.
-					 */
-					for (j = 0; j < alg_size; j++) {
-						if (alg[j] == kx) {
-							keep = 0;
-							break;
-						}
-					}
-				} else	/* CLIENT */
-					keep = 0;
-			}
+			if (server)
+				keep = check_server_params( session, kx, alg, alg_size);
 		}
+		
+		
 
 		memcpy( &cs.CipherSuite, &(*cipherSuites)[i].CipherSuite, 2);
 
@@ -2462,7 +2531,7 @@ int _gnutls_remove_unwanted_ciphersuites(gnutls_session session,
 
 /**
   * gnutls_handshake_set_max_packet_length - This function will set the maximum length of a handshake message
-  * @session: is a a &gnutls_session structure.
+  * @session: is a &gnutls_session structure.
   * @max: is the maximum number.
   *
   * This function will set the maximum size of a handshake message.
