@@ -26,42 +26,36 @@
 /* for message digests */
 
 typedef struct {
-    gcry_md_hd_t handle;
-    gnutls_mac_algorithm algorithm;
-    const void *key;
-    int keysize;
-} mac_hd_t_INT;
-typedef mac_hd_t_INT *mac_hd_t;
-typedef mac_hd_t GNUTLS_HASH_HANDLE;
+	gcry_md_hd_t handle;
+	gnutls_mac_algorithm algorithm;
+	const void* key;
+	int keysize;
+} GNUTLS_MAC_HANDLE_INT;
+typedef GNUTLS_MAC_HANDLE_INT* GNUTLS_MAC_HANDLE;
+typedef GNUTLS_MAC_HANDLE GNUTLS_HASH_HANDLE;
 
 #define GNUTLS_HASH_FAILED NULL
 #define GNUTLS_MAC_FAILED NULL
 
-mac_hd_t _gnutls_hmac_init(gnutls_mac_algorithm algorithm, const void *key,
-			   int keylen);
+GNUTLS_MAC_HANDLE _gnutls_hmac_init( gnutls_mac_algorithm algorithm, const void* key, int keylen);
 #define _gnutls_hmac_get_algo_len _gnutls_hash_get_algo_len
 #define _gnutls_hmac _gnutls_hash
-void _gnutls_hmac_deinit(mac_hd_t handle, void *digest);
+void _gnutls_hmac_deinit( GNUTLS_MAC_HANDLE handle, void* digest);
 
-mac_hd_t _gnutls_mac_init_ssl3(gnutls_mac_algorithm algorithm, void *key,
-			       int keylen);
-void _gnutls_mac_deinit_ssl3(mac_hd_t handle, void *digest);
+GNUTLS_MAC_HANDLE _gnutls_mac_init_ssl3( gnutls_mac_algorithm algorithm, void* key, int keylen);
+void _gnutls_mac_deinit_ssl3( GNUTLS_MAC_HANDLE handle, void* digest);
 
 GNUTLS_HASH_HANDLE _gnutls_hash_init(gnutls_mac_algorithm algorithm);
 int _gnutls_hash_get_algo_len(gnutls_mac_algorithm algorithm);
-int _gnutls_hash(GNUTLS_HASH_HANDLE handle, const void *text,
-		 size_t textlen);
-void _gnutls_hash_deinit(GNUTLS_HASH_HANDLE handle, void *digest);
+int _gnutls_hash(GNUTLS_HASH_HANDLE handle, const void* text, size_t textlen);
+void _gnutls_hash_deinit(GNUTLS_HASH_HANDLE handle, void* digest);
 
-int _gnutls_ssl3_generate_random(void *secret, int secret_len,
-				 void *random, int random_len, int bytes,
-				 opaque * ret);
-int _gnutls_ssl3_hash_md5(void *first, int first_len, void *second,
-			  int second_len, int ret_len, opaque * ret);
+int _gnutls_ssl3_generate_random(void *secret, int secret_len, void *random, int random_len, int bytes, opaque* ret);
+int _gnutls_ssl3_hash_md5(void *first, int first_len, 
+	void *second, int second_len, int ret_len, opaque* ret);
 
-void _gnutls_mac_deinit_ssl3_handshake(mac_hd_t handle, void *digest,
-				       opaque * key, uint32 key_size);
+void _gnutls_mac_deinit_ssl3_handshake(GNUTLS_MAC_HANDLE handle, void* digest, opaque* key, uint32 key_size);
 
 GNUTLS_HASH_HANDLE _gnutls_hash_copy(GNUTLS_HASH_HANDLE handle);
 
-#endif				/* GNUTLS_HASH_INT_H */
+#endif /* GNUTLS_HASH_INT_H */
