@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000 Nikos Mavroyanopoulos
+ * Copyright (C) 2004 Free Software Foundation
  *
  * This file is part of GNUTLS.
  *
@@ -190,7 +191,7 @@ int gnutls_protocol_set_priority(gnutls_session session, const int *list)
 int gnutls_certificate_type_set_priority(gnutls_session session,
 					 const int *list)
 {
-#ifdef HAVE_LIBOPENCDK
+#ifdef USE_OPENPGP
 
 	int num = 0, i;
 
