@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000 Nikos Mavroyanopoulos
+ * Copyright (C) 2004 Free Software Foundation
  *
  * This file is part of GNUTLS.
  *
@@ -52,7 +53,7 @@ int _gnutls_session_pack(gnutls_session session, gnutls_datum * packed_session)
 
 	if (packed_session==NULL) {
 		gnutls_assert();
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
 
@@ -127,7 +128,7 @@ int _gnutls_session_pack(gnutls_session session, gnutls_datum * packed_session)
 		}
 		break;
 	default:
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		return GNUTLS_E_INTERNAL_ERROR;
 
 	}
 
@@ -140,13 +141,12 @@ int _gnutls_session_pack(gnutls_session session, gnutls_datum * packed_session)
 	       data[packed_session->size - sizeof(SecurityParameters)],
 	       &session->security_parameters, sizeof(SecurityParameters));
 
-
 	return 0;
 }
 
 /* Returns the size needed to hold the current session.
  */
-int _gnutls_session_size( gnutls_session session)
+uint _gnutls_session_size( gnutls_session session)
 {
 	uint32 pack_size;
 
@@ -183,7 +183,7 @@ int _gnutls_session_unpack(gnutls_session session,
 
 	if (packed_session==NULL || packed_session->size == 0) {
 		gnutls_assert();
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		return GNUTLS_E_INTERNAL_ERROR;
 	}
 
 	if (session->key->auth_info != NULL) {
@@ -287,7 +287,7 @@ int _gnutls_session_unpack(gnutls_session session,
 		break;
 	default:
 		gnutls_assert();
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		return GNUTLS_E_INTERNAL_ERROR;
 
 	}
 
