@@ -111,7 +111,7 @@ _asn1_tag_der(unsigned char class,unsigned int tag_value,unsigned char *ans,int
   int k;
   unsigned char temp[SIZEOF_UNSIGNED_INT];
 
-  if(tag_value<30){
+  if(tag_value<31){
     /* short form */
     ans[0]=(class&0xE0) + ((unsigned char)(tag_value&0x1F));
     *ans_len=1;
@@ -592,8 +592,8 @@ _asn1_ordering_set(unsigned char *der,node_asn *node)
 	if (temp==NULL) return;
 	
 	memcpy(temp,der+counter,p_vet->end-counter);
-	memmove(der+counter,der+p_vet->end,p2_vet->end-p_vet->end);
-	memcpy(der+p_vet->end,temp,p_vet->end-counter);
+	memcpy(der+counter,der+p_vet->end,p2_vet->end-p_vet->end);
+	memcpy(der+counter+p2_vet->end-p_vet->end,temp,p_vet->end-counter);
 	_asn1_afree(temp);
 	
 	tag=p_vet->value;
@@ -693,9 +693,9 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
 	temp=(unsigned char *)_asn1_alloca(p_vet->end-counter);
 	if (temp==NULL) return;
 	
-	memcpy(temp,der+counter,p_vet->end-counter);
-	memmove(der+counter,der+p_vet->end,p2_vet->end-p_vet->end);
-	memcpy(der+p_vet->end,temp,p_vet->end-counter);
+	memcpy(temp,der+counter,(p_vet->end)-counter);
+	memcpy(der+counter,der+(p_vet->end),(p2_vet->end)-(p_vet->end));
+	memcpy(der+counter+(p2_vet->end)-(p_vet->end),temp,(p_vet->end)-counter);
 	_asn1_afree(temp);
 	
 	p_vet->end=counter+(p2_vet->end-p_vet->end);
@@ -717,7 +717,7 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
   * asn1_der_coding - Creates the DER encoding for the NAME structure
   * @element: pointer to an ASN1 element
   * @name: the name of the structure you want to encode (it must be inside *POINTER).
-  * @der: vector that will contain the DER encoding. DER must be a pointer to memory cells already allocated.
+  * @ider: vector that will contain the DER encoding. DER must be a pointer to memory cells already allocated.
   * @len: number of bytes of *der: der[0]..der[len-1], Initialy holds the sizeof of der vector.
   * @errorDescription : return the error description or an empty string if success.
   * Description:
@@ -737,13 +737,14 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
   *
   **/
 asn1_retCode 
-asn1_der_coding(ASN1_TYPE element,const char *name,unsigned char *der,int *len,
+asn1_der_coding(ASN1_TYPE element,const char *name,void *ider,int *len,
                 char *ErrorDescription)
 {
   node_asn *node,*p,*p2;
   char temp[SIZEOF_UNSIGNED_LONG_INT*3+1];
   int counter,counter_old,len2,len3,move,max_len,max_len_old;
   asn1_retCode ris;
+  unsigned char* der = ider;
 
   node=_asn1_find_node(element,name);
   if(node==NULL) return ASN1_ELEMENT_NOT_FOUND;
