@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2001 Nikos Mavroyanopoulos
+ * Copyright (C) 2001,2003 Nikos Mavroyanopoulos
+ * Copyright (C) 2004 Free Software Foundation
  *
  * This file is part of GNUTLS.
  *
@@ -73,19 +74,20 @@ int _gnutls_srp_gx(opaque * text, size_t textsize, opaque ** result, GNUTLS_MPI
 
 
 /****************
- * Choose a random value b and calculate B = (v + g^b) % N.
+ * Choose a random value b and calculate B = (k*v + g^b) % N.
+ * where k == SHA1(N|g)
  * Return: B and if ret_b is not NULL b.
  */
 GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GNUTLS_MPI v)
 {
-	GNUTLS_MPI tmpB, tmpV;
-	GNUTLS_MPI b, B;
+	GNUTLS_MPI tmpB = NULL, tmpV = NULL;
+	GNUTLS_MPI b = NULL, B = NULL, k = NULL;
 	int bits;
 
-	/* calculate:  B = (3v + g^b) % N 
+	/* calculate:  B = (k*v + g^b) % N 
 	 */
 	bits = _gnutls_mpi_get_nbits(n);
-	b = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	b = _gnutls_mpi_snew(bits);
 	if (b==NULL) {
 		gnutls_assert();
 		return NULL;
@@ -94,34 +96,35 @@ GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GN
 	tmpV = _gnutls_mpi_alloc_like(n);
 
 	if (tmpV == NULL) {
-		_gnutls_mpi_release(&b);
-		return NULL;
+		gnutls_assert();
+		goto error;
 	}
 	
 	_gnutls_mpi_randomize(b, bits, GCRY_STRONG_RANDOM);
 
-	tmpB = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	tmpB = _gnutls_mpi_snew(bits);
 	if (tmpB==NULL) {
 		gnutls_assert();
-		_gnutls_mpi_release( &b);
-		_gnutls_mpi_release(&tmpV);
-		return NULL;
+		goto error;
 	}
 
-	B = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
-	if (tmpB==NULL) {
+	B = _gnutls_mpi_snew(bits);
+	if (B==NULL) {
 		gnutls_assert();
-		_gnutls_mpi_release( &b);
-		_gnutls_mpi_release( &tmpB);
-		_gnutls_mpi_release(&tmpV);
-		return NULL;
+		goto error;
 	}
 
-	_gnutls_mpi_mul_ui(tmpV, v, 3);
+	k = _gnutls_calc_srp_u( n, g);
+	if (k == NULL) {
+		gnutls_assert();
+		goto error;
+	}
 
+	_gnutls_mpi_mulm(tmpV, k, v, n);
 	_gnutls_mpi_powm(tmpB, g, b, n);
 	_gnutls_mpi_addm(B, tmpV, tmpB, n);
 
+	_gnutls_mpi_release( &k);
 	_gnutls_mpi_release(&tmpB);
 	_gnutls_mpi_release(&tmpV);
 
@@ -131,15 +134,24 @@ GNUTLS_MPI _gnutls_calc_srp_B(GNUTLS_MPI * ret_b, GNUTLS_MPI g, GNUTLS_MPI n, GN
 		_gnutls_mpi_release(&b);
 
 	return B;
+
+error:
+	_gnutls_mpi_release( &b);
+	_gnutls_mpi_release( &B);
+	_gnutls_mpi_release( &k);
+	_gnutls_mpi_release( &tmpB);
+	_gnutls_mpi_release(&tmpV);
+	return NULL;
 }
 
+/* This calculates the SHA1(A | B)
+ */
 GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B)
 {
 	size_t b_size, a_size;
 	opaque *holder, hd[MAX_HASH_SIZE];
 	size_t holder_size, hash_size;
 	GNUTLS_HASH_HANDLE td;
-	uint32 u;
 	int ret;
 	GNUTLS_MPI res;
 
@@ -162,8 +174,8 @@ GNUTLS_MPI _gnutls_calc_srp_u(GNUTLS_MPI A, GNUTLS_MPI B)
 	}
 	_gnutls_hash(td, holder, holder_size);
 	_gnutls_hash_deinit(td, hd);
-	
-	/* convert the bytes of hd to MPI.
+
+	/* convert the bytes of hd to integer
 	 */
 	hash_size = 20; /* SHA */
 	ret = _gnutls_mpi_scan( &res, hd, &hash_size);
@@ -199,13 +211,15 @@ GNUTLS_MPI _gnutls_calc_srp_S1(GNUTLS_MPI A, GNUTLS_MPI b, GNUTLS_MPI u, GNUTLS_
 	_gnutls_mpi_mulm(tmp2, A, tmp1, n);
 	_gnutls_mpi_powm(S, tmp2, b, n);
 
-	return S;
-
-	freeall:
 	_gnutls_mpi_release(&tmp1);
 	_gnutls_mpi_release(&tmp2);
 
-	return NULL;
+	return S;
+
+	freeall:
+		_gnutls_mpi_release(&tmp1);
+		_gnutls_mpi_release(&tmp2);
+		return NULL;
 }
 
 /* A = g^a % N 
@@ -218,7 +232,7 @@ GNUTLS_MPI _gnutls_calc_srp_A(GNUTLS_MPI * a, GNUTLS_MPI g, GNUTLS_MPI n)
 	int bits;
 
 	bits = _gnutls_mpi_get_nbits(n);
-	tmpa = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	tmpa = _gnutls_mpi_snew(bits);
 	if (tmpa==NULL) {
 		gnutls_assert();
 		return NULL;
@@ -226,7 +240,7 @@ GNUTLS_MPI _gnutls_calc_srp_A(GNUTLS_MPI * a, GNUTLS_MPI g, GNUTLS_MPI n)
 	
 	_gnutls_mpi_randomize(tmpa, bits, GCRY_STRONG_RANDOM);
 
-	A = _gnutls_mpi_new(bits);	/* FIXME: allocate in secure memory */
+	A = _gnutls_mpi_snew(bits);
 	if (A==NULL) {
 		gnutls_assert();
 		_gnutls_mpi_release( &tmpa);
@@ -245,8 +259,8 @@ GNUTLS_MPI _gnutls_calc_srp_A(GNUTLS_MPI * a, GNUTLS_MPI g, GNUTLS_MPI n)
 /* generate x = SHA(s | SHA(U | ":" | p))
  * The output is exactly 20 bytes
  */
-int _gnutls_calc_srp_sha(char *username, char *password, opaque * salt,
-			   int salt_size, int *size, void* digest)
+int _gnutls_calc_srp_sha(const char *username, const char *password, opaque * salt,
+			   int salt_size, size_t *size, void* digest)
 {
 	GNUTLS_HASH_HANDLE td;
 	opaque res[MAX_HASH_SIZE];
@@ -291,7 +305,7 @@ int _gnutls_calc_srp_x(char *username, char *password, opaque * salt,
 GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_MPI a, GNUTLS_MPI u, GNUTLS_MPI n)
 {
 	GNUTLS_MPI S=NULL, tmp1=NULL, tmp2=NULL;
-	GNUTLS_MPI tmp4=NULL, tmp3=NULL;
+	GNUTLS_MPI tmp4=NULL, tmp3=NULL, k = NULL;
 
 	S = _gnutls_mpi_alloc_like(n);
 	if (S==NULL)
@@ -303,9 +317,15 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
 	if (tmp1 == NULL || tmp2 == NULL || tmp3 == NULL) {
 		goto freeall;
 	}
+	
+	k = _gnutls_calc_srp_u( n, g);
+	if (k==NULL) {
+		gnutls_assert();
+		goto freeall;
+	}
 
 	_gnutls_mpi_powm(tmp1, g, x, n); /* g^x */
-	_gnutls_mpi_mul_ui(tmp3, tmp1, 3); /* 3*g^x */
+	_gnutls_mpi_mulm(tmp3, tmp1, k, n); /* k*g^x mod n */
 	_gnutls_mpi_subm(tmp2, B, tmp3, n);
 
 	tmp4 = _gnutls_mpi_alloc_like(n);
@@ -315,18 +335,23 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
 	_gnutls_mpi_mul(tmp1, u, x);
 	_gnutls_mpi_add(tmp4, a, tmp1);
 	_gnutls_mpi_powm(S, tmp2, tmp4, n);
-	
-	return S;
-
-	freeall:
 
 	_gnutls_mpi_release(&tmp1);
 	_gnutls_mpi_release(&tmp2);
 	_gnutls_mpi_release(&tmp3);
 	_gnutls_mpi_release(&tmp4);
-	_gnutls_mpi_release(&S);
+	_gnutls_mpi_release(&k);
+	
+	return S;
 
-	return NULL;
+	freeall:
+		_gnutls_mpi_release(&tmp1);
+		_gnutls_mpi_release(&tmp2);
+		_gnutls_mpi_release(&tmp3);
+		_gnutls_mpi_release(&tmp4);
+		_gnutls_mpi_release(&S);
+		_gnutls_mpi_release(&k);
+		return NULL;
 }
 
 /**
@@ -334,8 +359,8 @@ GNUTLS_MPI _gnutls_calc_srp_S2(GNUTLS_MPI B, GNUTLS_MPI g, GNUTLS_MPI x, GNUTLS_
   * @sc: is an &gnutls_srp_client_credentials structure.
   *
   * This structure is complex enough to manipulate directly thus
-  * this helper function is provided in order to free (deallocate)
-  * the structure.
+  * this helper function is provided in order to free (deallocate) it.
+  *
   **/
 void gnutls_srp_free_client_credentials( gnutls_srp_client_credentials sc) {
 	gnutls_free( sc->username);
@@ -348,8 +373,9 @@ void gnutls_srp_free_client_credentials( gnutls_srp_client_credentials sc) {
   * @sc: is a pointer to an &gnutls_srp_server_credentials structure.
   *
   * This structure is complex enough to manipulate directly thus
-  * this helper function is provided in order to allocate
-  * the structure.
+  * this helper function is provided in order to allocate it.
+  *
+  * Returns 0 on success.
   **/
 int gnutls_srp_allocate_client_credentials( gnutls_srp_client_credentials *sc) {
 	*sc = gnutls_calloc( 1, sizeof(SRP_CLIENT_CREDENTIALS_INT));
@@ -365,8 +391,14 @@ int gnutls_srp_allocate_client_credentials( gnutls_srp_client_credentials *sc) {
   * @username: is the user's userid
   * @password: is the user's password
   *
+  * This function sets the username and password, in a gnutls_srp_client_credentials structure.
+  * Those will be used in SRP authentication. @username and @password should be ASCII
+  * strings or UTF-8 strings prepared using the "SASLprep" profile of "stringprep".
+  *
+  * Returns 0 on success.
   **/
-int gnutls_srp_set_client_credentials( gnutls_srp_client_credentials res, char *username, char * password) {
+int gnutls_srp_set_client_credentials( gnutls_srp_client_credentials res, char *username, char * password) 
+{
 
 	if (username==NULL || password == NULL) {
 		gnutls_assert();
@@ -390,8 +422,8 @@ int gnutls_srp_set_client_credentials( gnutls_srp_client_credentials res, char *
   * @sc: is an &gnutls_srp_server_credentials structure.
   *
   * This structure is complex enough to manipulate directly thus
-  * this helper function is provided in order to free (deallocate)
-  * the structure.
+  * this helper function is provided in order to free (deallocate) it.
+  *
   **/
 void gnutls_srp_free_server_credentials( gnutls_srp_server_credentials sc) {
 int i;
@@ -410,8 +442,9 @@ int i;
   * @sc: is a pointer to an &gnutls_srp_server_credentials structure.
   *
   * This structure is complex enough to manipulate directly thus
-  * this helper function is provided in order to allocate
-  * the structure.
+  * this helper function is provided in order to allocate it.
+  * 
+  * Returns 0 on success.
   **/
 int gnutls_srp_allocate_server_credentials( gnutls_srp_server_credentials *sc) {
 	*sc = gnutls_calloc( 1, sizeof(SRP_SERVER_CREDENTIALS_INT));
@@ -438,8 +471,15 @@ FILE* fd;
   * @password_file: is the SRP password file (tpasswd)
   * @password_conf_file: is the SRP password conf file (tpasswd.conf)
   *
+  * This function sets the password files, in a gnutls_srp_server_credentials structure.
+  * Those password files hold usernames and verifiers and will be used for SRP authentication.
+  *
+  * Returns 0 on success.
+  *
   **/
-int gnutls_srp_set_server_credentials_file( gnutls_srp_server_credentials res, char *password_file, char * password_conf_file) {
+int gnutls_srp_set_server_credentials_file( gnutls_srp_server_credentials res, 
+	const char *password_file, const char * password_conf_file) 
+{
 int i;
 	
 	if (password_file==NULL || password_conf_file==NULL) {
@@ -493,21 +533,24 @@ int i;
 	return 0;
 }
 
+#define gnutls_srp_server_select_function srp_server_select_func
+
 /**
   * gnutls_srp_server_set_select_function - Used to set a callback to assist in selecting the proper password file
   * @session: is a &gnutls_session structure.
   * @func: is the callback function
   *
-  * The callback's function form is:
+  * This function sets a callback to assist in selecting the proper password file,
+  * in case there are more than one. The callback's function form is:
   * int (*callback)(gnutls_session, const char** pfiles, const char** pconffiles, int npfiles);
   *
-  * 'pfiles' contains 'npfiles' char* structures which hold
-  * the password file name. 'pconffiles' contain the corresponding
+  * @pfiles contains @npfiles char* structures which hold
+  * the password file name. @pconffiles contain the corresponding
   * conf files.
   *
   * This function specifies what we, in case of a server, are going
   * to do when we have to use a password file. If this callback
-  * function is not provided then gnutls will automaticaly select the
+  * function is not provided then gnutls will automatically select the
   * first password file
   *
   * In case the callback returned a negative number then gnutls will
@@ -519,7 +562,7 @@ int i;
   *
   **/
 void gnutls_srp_server_set_select_function(gnutls_session session,
-					     srp_server_select_func
+					     gnutls_srp_server_select_function
 					     * func)
 {
 	session->internals.server_srp_callback = func;
@@ -530,25 +573,27 @@ void gnutls_srp_server_set_select_function(gnutls_session session,
   * @cred: is a &gnutls_srp_server_credentials structure.
   * @func: is the callback function
   *
+  * This function can be used to set a callback to retrieve the user's SRP credentials.
   * The callback's function form is:
   * int (*callback)(gnutls_session, const char* username,
   *  gnutls_datum* salt, gnutls_datum *verifier, gnutls_datum* g,
   *  gnutls_datum* n);
   *
-  * 'username' contains the actual username. 
-  * The 'salt', 'verifier', 'generator' and 'prime' must be filled
-  * in using the gnutls_malloc().
+  * @username contains the actual username. 
+  * The @salt, @verifier, @generator and @prime must be filled
+  * in using the gnutls_malloc(). For convenience @prime and @generator
+  * may also be one of the static parameters defined in extra.h.
   *
   * In case the callback returned a negative number then gnutls will
   * assume that the username does not exist.
   *
-  * In order to prevent allowing an attack to guess valid usernames,
-  * if username does not exist, the g, and n values should be filled in 
-  * using a random user's parameters. In that case the callback should
+  * In order to prevent attackers from guessing valid usernames,
+  * if a user does not exist, g and n values should be filled in 
+  * using a random user's parameters. In that case the callback must
   * return the special value (1).
   *
   * The callback function will only be called once per handshake.
-  * The callback function should return 0 on success.
+  * The callback function should return 0 on success, while
   * -1 indicates an error.
   *
   **/
@@ -559,6 +604,44 @@ void gnutls_srp_set_server_credentials_function(
 	cred->pwd_callback = func;
 }
 
+/**
+  * gnutls_srp_set_client_credentials_function - Used to set a callback to retrieve the username and password
+  * @cred: is a &gnutls_srp_server_credentials structure.
+  * @func: is the callback function
+  *
+  * This function can be used to set a callback to retrieve the username and
+  * password for client SRP authentication.
+  * The callback's function form is:
+  * int (*callback)(gnutls_session, unsigned int times, char** username,
+  *  char** password);
+  *
+  * The @username and @password must be allocated using gnutls_malloc().
+  * @times will be 0 the first time called, and 1 the second.
+  * @username and @password should be ASCII strings or UTF-8 strings 
+  * prepared using the "SASLprep" profile of "stringprep".
+  *
+  * The callback function will be called once or twice per handshake.
+  * The first time called, is before the ciphersuite is negotiated.
+  * At that time if the callback returns a negative error code,
+  * the callback will be called again if SRP has been
+  * negotiated. This uses a special TLS-SRP idiom in order to avoid
+  * asking the user for SRP password and username if the server does
+  * not support SRP.
+  * 
+  * The callback should not return a negative error code the second
+  * time called, since the handshake procedure will be aborted.
+  *
+  * The callback function should return 0 on success.
+  * -1 indicates an error.
+  *
+  **/
+void gnutls_srp_set_client_credentials_function(
+	gnutls_srp_client_credentials cred, 
+	gnutls_srp_client_credentials_function * func)
+{
+	cred->get_function = func;
+}
+
 
 /**
   * gnutls_srp_server_get_username - This function returns the username of the peer
@@ -586,23 +669,27 @@ const char *gnutls_srp_server_get_username(gnutls_session session)
   * @username: is the user's name
   * @password: is the user's password
   * @salt: should be some randomly generated bytes
-  * @g: is a generator
-  * @n: is a prime
+  * @generator: is the generator of the group
+  * @prime: is the group's prime
   * @res: where the verifier will be stored.
   *
   * This function will create an SRP verifier, as specified in RFC2945.
-  * The prime and generator may be generated using the gnutls_dh_params_generate()
-  * function. The verifier will be allocated with 'malloc' and will be stored in 'res' in binary format.
+  * The @prime and @generator should be one of the static parameters defined
+  * in gnutls/extra.h or may be generated using the GCRYPT functions
+  * gcry_prime_generate() and gcry_prime_group_generator().
+  * The verifier will be allocated with @malloc and will be stored in @res using 
+  * binary format.
   *
   **/
-int gnutls_srp_verifier( char* username, char* password, const gnutls_datum *salt, 
-	const gnutls_datum* g, const gnutls_datum* n, 
+int gnutls_srp_verifier( const char* username, const char* password, 
+	const gnutls_datum *salt, 
+	const gnutls_datum* generator, const gnutls_datum* prime, 
 	gnutls_datum * res)
 {
 GNUTLS_MPI _n, _g;
-int ret, digest_size = 20;
+int ret;
+size_t digest_size = 20, size;
 opaque digest[20];
-size_t size;
 
 	ret = _gnutls_calc_srp_sha( username, password, salt->data,
 			   salt->size, &digest_size, digest);
@@ -611,14 +698,14 @@ size_t size;
 		return ret;
 	}
 
-	size = n->size;
-	if (_gnutls_mpi_scan(&_n, n->data, &size)) {
+	size = prime->size;
+	if (_gnutls_mpi_scan(&_n, prime->data, &size)) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
-	size = g->size;
-	if (_gnutls_mpi_scan(&_g, g->data, &size)) {
+	size = generator->size;
+	if (_gnutls_mpi_scan(&_g, generator->data, &size)) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
@@ -633,36 +720,4 @@ size_t size;
 	return 0;
 }
 
-/* FIXME: Functions for backwards compatibility 
- */
-
-void gnutls_srp_free_client_cred( gnutls_srp_client_credentials sc) {
-	return gnutls_srp_free_client_credentials( sc);
-}
-
-void gnutls_srp_free_server_cred( gnutls_srp_server_credentials sc) {
-	return gnutls_srp_free_server_credentials( sc);
-}
-
-int gnutls_srp_allocate_client_cred( gnutls_srp_client_credentials *sc) {
-	return gnutls_srp_allocate_client_credentials( sc);
-}
-
-int gnutls_srp_allocate_server_cred( gnutls_srp_server_credentials *sc) 
-{
-	return gnutls_srp_allocate_server_credentials( sc);
-}
-
-int gnutls_srp_set_server_cred_file( gnutls_srp_server_credentials res, char *password_file, char * password_conf_file) 
-{
-	return gnutls_srp_set_server_credentials_file( res, password_file, password_conf_file);
-}
-
-int gnutls_srp_set_client_cred( gnutls_srp_client_credentials res, char *username, char * password) 
-{
-	return gnutls_srp_set_client_credentials( res, username, password);
-}
-
-
-
 #endif /* ENABLE_SRP */
