@@ -19,6 +19,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
+
+#ifdef ENABLE_PKI
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -130,3 +134,5 @@ int generate_prime(int bits)
 
 	return 0;
 }
+
+#endif
